@@ -96,6 +96,51 @@ def depthwise_conv2d_no_bias(inputs, kernel_size, strides=1, padding="VALID", us
     )(inputs)
 
 
+def deep_stem(inputs, stem_width, activation="relu", last_strides=1, name=None):
+    nn = conv2d_no_bias(inputs, stem_width // 2, 3, strides=2, padding="same", name=name and name + "1_")
+    nn = batchnorm_with_activation(nn, activation=activation, name=name and name + "1_")
+    nn = conv2d_no_bias(nn, stem_width // 2, 3, strides=1, padding="same", name=name and name + "2_")
+    nn = batchnorm_with_activation(nn, activation=activation, name=name and name + "2_")
+    nn = conv2d_no_bias(nn, stem_width, 3, strides=last_strides, padding="same", name=name and name + "3_")
+    return nn
+
+
+def quad_stem(inputs, stem_width, activation="relu", stem_act=False, last_strides=2, name=None):
+    nn = conv2d_no_bias(inputs, stem_width // 8, 3, strides=2, padding="same", name=name and name + "1_")
+    if stem_act:
+        nn = batchnorm_with_activation(nn, activation=activation, name=name and name + "1_")
+    nn = conv2d_no_bias(nn, stem_width // 4, 3, strides=1, padding="same", name=name and name + "2_")
+    if stem_act:
+        nn = batchnorm_with_activation(nn, activation=activation, name=name and name + "2_")
+    nn = conv2d_no_bias(nn, stem_width // 2, 3, strides=1, padding="same", name=name and name + "3_")
+    nn = batchnorm_with_activation(nn, activation=activation, name=name and name + "3_")
+    nn = conv2d_no_bias(nn, stem_width, 3, strides=last_strides, padding="same", name=name and name + "4_")
+    return nn
+
+
+def tiered_stem(inputs, stem_width, activation="relu", last_strides=1, name=None):
+    nn = conv2d_no_bias(inputs, 3 * stem_width // 8, 3, strides=2, padding="same", name=name and name + "1_")
+    nn = batchnorm_with_activation(nn, activation=activation, name=name and name + "1_")
+    nn = conv2d_no_bias(nn, stem_width // 2, 3, strides=1, padding="same", name=name and name + "2_")
+    nn = batchnorm_with_activation(nn, activation=activation, name=name and name + "2_")
+    nn = conv2d_no_bias(nn, stem_width, 3, strides=last_strides, padding="same", name=name and name + "3_")
+    return nn
+
+
+def output_block(inputs, num_features=0, activation="relu", num_classes=1000, drop_rate=0, classifier_activation="softmax"):
+    nn = inputs
+    if num_features > 0:  # efficientnet like
+        nn = conv2d_no_bias(nn, num_features, 1, strides=1, name="features_")
+        nn = batchnorm_with_activation(nn, activation=activation, name="features_")
+
+    if num_classes > 0:
+        nn = keras.layers.GlobalAveragePooling2D(name="avg_pool")(nn)
+        if drop_rate > 0:
+            nn = keras.layers.Dropout(drop_rate, name="head_drop")(nn)
+        nn = keras.layers.Dense(num_classes, dtype="float32", activation=classifier_activation, name="predictions")(nn)
+    return nn
+
+
 def se_module(inputs, se_ratio=0.25, divisor=8, activation="relu", use_bias=True, name=None):
     """ Squeeze-and-Excitation block, arxiv: https://arxiv.org/pdf/1709.01507.pdf """
     channel_axis = 1 if K.image_data_format() == "channels_first" else -1
@@ -112,14 +157,14 @@ def se_module(inputs, se_ratio=0.25, divisor=8, activation="relu", use_bias=True
     return keras.layers.Multiply(name=name and name + "out")([inputs, se])
 
 
-def eca_module(inputs, gamma=2., beta=1., name=None, **kwargs):
+def eca_module(inputs, gamma=2.0, beta=1.0, name=None, **kwargs):
     """ Efficient Channel Attention block, arxiv: https://arxiv.org/pdf/1910.03151.pdf """
     channel_axis = 1 if K.image_data_format() == "channels_first" else -1
     h_axis, w_axis = [2, 3] if K.image_data_format() == "channels_first" else [1, 2]
 
     filters = inputs.shape[channel_axis]
     beta, gamma = float(beta), float(gamma)
-    tt = int((tf.math.log(float(filters)) / tf.math.log(2.) + beta) / gamma)
+    tt = int((tf.math.log(float(filters)) / tf.math.log(2.0) + beta) / gamma)
     kernel_size = max(tt if tt % 2 else tt + 1, 3)
     pad = kernel_size // 2
 
@@ -133,6 +178,11 @@ def eca_module(inputs, gamma=2., beta=1., name=None, **kwargs):
     return keras.layers.Multiply(name=name and name + "out")([inputs, nn])
 
 
+def drop_connect_rates_split(num_blocks, start=0.0, end=0.0):
+    drop_connect_rates = tf.split(tf.linspace(start, end, sum(num_blocks)), num_blocks)
+    return [ii.numpy().tolist() for ii in drop_connect_rates]
+
+
 def drop_block(inputs, drop_rate=0, name=None):
     """ Stochastic Depth block by Dropout, arxiv: https://arxiv.org/abs/1603.09382 """
     if drop_rate > 0:
@@ -177,6 +227,21 @@ def make_divisible(vv, divisor=4, min_value=None):
 
 
 def tpu_extract_patches_overlap_1(inputs, sizes=3, strides=2, rates=1, padding="SAME", name=None):
+    """For issue https://github.com/leondgarse/keras_cv_attention_models/issues/8,
+    `tf.image.extract_patches` NOT working for TPU.
+    `overlap_1` means `1 < sizes / strides <= 2`.
+    `rates` and `name` not used, just keeping same perameters with `tf.image.extract_patche`.
+
+    input: `[batch, height, width, channel]`.
+    output: `[batch, height // strides,  width // strides, sizes * sizes * channel]`.
+
+    Examples:
+
+    >>> from keras_cv_attention_models import attention_layers
+    >>> inputs = tf.ones([1, 64, 27, 192])
+    >>> print(attention_layers.tpu_extract_patches_overlap_1(inputs, sizes=3, strides=2).shape)
+    # (1, 32, 14, 1728)
+    """
     # kernel_size, strides = 3, 2
     # inputs = np.random.uniform(size=[1, 64, 28, 192])
     kernel_size = sizes[1] if isinstance(sizes, (list, tuple)) else sizes
@@ -196,30 +261,29 @@ def tpu_extract_patches_overlap_1(inputs, sizes=3, strides=2, rates=1, padding="
     # print(f"{ww = }, {hh = }, {cc = }, {num_patches_hh = }, {num_patches_ww = }, {valid_hh = }, {valid_ww = }, {overlap_s = }")
     # ww = 30, hh = 66, cc = 192, num_patches_hh = 32, num_patches_ww = 14, valid_hh = 64, valid_ww = 28, overlap_s = 1
 
-    center = tf.reshape(pad_inputs[:, :valid_hh, :valid_ww, :], temp_shape) # (1, 32, 2, 14, 2, 192)
-    ww_overlap = tf.reshape(pad_inputs[:, :valid_hh, overlap_s:valid_ww + overlap_s, :], temp_shape)  # (1, 32, 2, 14, 2, 192)
-    hh_overlap = tf.reshape(pad_inputs[:, overlap_s:valid_hh + overlap_s, :valid_ww, :], temp_shape)  # (1, 32, 2, 14, 2, 192)
-    corner_overlap = tf.reshape(pad_inputs[:, overlap_s:valid_hh + overlap_s, overlap_s:valid_ww + overlap_s, :], temp_shape) # (1, 32, 2, 14, 2, 192)
+    center = tf.reshape(pad_inputs[:, :valid_hh, :valid_ww, :], temp_shape)  # (1, 32, 2, 14, 2, 192)
+    ww_overlap = tf.reshape(pad_inputs[:, :valid_hh, overlap_s : valid_ww + overlap_s, :], temp_shape)  # (1, 32, 2, 14, 2, 192)
+    hh_overlap = tf.reshape(pad_inputs[:, overlap_s : valid_hh + overlap_s, :valid_ww, :], temp_shape)  # (1, 32, 2, 14, 2, 192)
+    corner_overlap = tf.reshape(pad_inputs[:, overlap_s : valid_hh + overlap_s, overlap_s : valid_ww + overlap_s, :], temp_shape)  # (1, 32, 2, 14, 2, 192)
     # print(f"{center.shape = }, {corner_overlap.shape = }")
     # center.shape = TensorShape([1, 32, 2, 14, 2, 192]), corner_overlap.shape = TensorShape([1, 32, 2, 14, 2, 192])
     # print(f"{ww_overlap.shape = }, {hh_overlap.shape = }")
     # ww_overlap.shape = TensorShape([1, 32, 2, 14, 2, 192]), hh_overlap.shape = TensorShape([1, 32, 2, 14, 2, 192])
 
-    aa = tf.concat([center, ww_overlap[:, :, :, :, -overlap_s:, :]], axis=4)    # (1, 32, 2, 14, 3, 192)
-    bb = tf.concat([hh_overlap[:, :, -overlap_s:, :, :, :], corner_overlap[:, :, -overlap_s:, :, -overlap_s:, :]], axis=4)    # (1, 32, 1, 14, 3, 192)
-    out = tf.concat([aa, bb], axis=2)  # (1, 32, 3, 14, 3, 192)
-    # print(f"{aa.shape = }, {bb.shape = }, {out.shape = }")
-    # aa.shape = TensorShape([1, 32, 2, 14, 3, 192]), bb.shape = TensorShape([1, 32, 1, 14, 3, 192]), out.shape = TensorShape([1, 32, 3, 14, 3, 192])
+    center_ww = tf.concat([center, ww_overlap[:, :, :, :, -overlap_s:, :]], axis=4)  # (1, 32, 2, 14, 3, 192)
+    hh_corner = tf.concat([hh_overlap[:, :, -overlap_s:, :, :, :], corner_overlap[:, :, -overlap_s:, :, -overlap_s:, :]], axis=4)  # (1, 32, 1, 14, 3, 192)
+    out = tf.concat([center_ww, hh_corner], axis=2)  # (1, 32, 3, 14, 3, 192)
+    # print(f"{center_ww.shape = }, {hh_corner.shape = }, {out.shape = }")
+    # center_ww.shape = TensorShape([1, 32, 2, 14, 3, 192]), hh_corner.shape = TensorShape([1, 32, 1, 14, 3, 192]), out.shape = TensorShape([1, 32, 3, 14, 3, 192])
 
-    out = tf.transpose(out, [0, 1, 3, 2, 4, 5]) # [1, 32, 14, 3, 3, 192]
+    out = tf.transpose(out, [0, 1, 3, 2, 4, 5])  # [batch, height, width, kernel, kernel, channel], [1, 32, 14, 3, 3, 192]
     # print(f"{out.shape = }")
     # out.shape = TensorShape([1, 32, 14, 3, 3, 192])
     return tf.reshape(out, [-1, num_patches_hh, num_patches_ww, kernel_size * kernel_size * cc])
 
 
 def tpu_compatible_extract_patches_overlap_1(inputs, sizes=3, strides=2, rates=1, padding="SAME", name=None):
-    """ For issue https://github.com/leondgarse/keras_cv_attention_models/issues/8, `tf.image.extract_patches` NOT working for TPU """
-    if len(tf.config.experimental.list_logical_devices('TPU')) == 0:
+    if len(tf.config.experimental.list_logical_devices("TPU")) == 0:
         return tf.image.extract_patches(inputs, sizes, strides, rates, padding, name)
     else:
         return tpu_extract_patches_overlap_1(inputs, sizes, strides, rates, padding, name)
