@@ -2,16 +2,16 @@ from keras_cv_attention_models.aotnet import AotNet
 from keras_cv_attention_models.download_and_load import reload_model_weights
 
 PRETRAINED_DICT = {
-    "resnet50d": {"imagenet": "deca3680b88300904a09d450e9a8c526"},
-    "resnet101d": {"imagenet": "201128b7bf68371399fe134c5e07f3db"},
-    "resnet152d": {"imagenet": "1e1823e0c2cf0f7031bc7abc0c9c97ba"},
-    "resnet200d": {"imagenet": "39d9050953e8d4fe9c672620542dd24d"},
+    "resnet50d": {"imagenet": "1b71933a82b058ba1e605ee5c01f64b2"},
+    "resnet101d": {"imagenet": "79b075be5cf222cff2bced7a5a117623"},
+    "resnet152d": {"imagenet": "0a15299b9abe1fee3ae06d9a59d13a3f"},
+    "resnet200d": {"imagenet": "b5961494e0072c342b838c77ef52ddc5"},
 }
 
 
-def ResNetD(num_blocks, input_shape=(224, 224, 3), pretrained="imagenet", deep_stem=True, stem_width=64, strides=2, avg_pool_down=True, **kwargs):
+def ResNetD(num_blocks, input_shape=(224, 224, 3), pretrained="imagenet", stem_type="deep", strides=2, avg_pool_down=True, **kwargs):
     strides = strides if isinstance(strides, (list, tuple)) else [1, 2, 2, strides]
-    model = AotNet(num_blocks, input_shape=input_shape, deep_stem=deep_stem, stem_width=stem_width, strides=strides, avg_pool_down=avg_pool_down, **kwargs)
+    model = AotNet(num_blocks, input_shape=input_shape, stem_type=stem_type, strides=strides, avg_pool_down=avg_pool_down, **kwargs)
     reload_model_weights(model, pretrained_dict=PRETRAINED_DICT, sub_release="resnet_family", input_shape=input_shape, pretrained=pretrained)
     return model
 
